@@ -12,7 +12,7 @@ import os
 import socket
 import traceback
 from pyasn1.type import univ
-from pysnmp.proto import rfc1902, rfc1905
+from pysnmp.proto import rfc1905
 from pysnmp.entity import engine, config
 from pysnmp.carrier.asynsock.dgram import udp
 try:
@@ -25,7 +25,6 @@ except ImportError:
     unix = None
 from pysnmp.entity.rfc3413 import cmdgen
 from pysnmp import debug
-from snmpsim import __version__
 from snmpsim.record import snmprec
 from snmpsim import confdir, error, log
 
@@ -66,13 +65,40 @@ privProtocols = {
   'NONE': config.usmNoPrivProtocol
 }
 
-helpMessage = 'Usage: %s [--help] [--debug=<category>] [--logging-method=<method|?>] [--version=<1|2c|3>] [--community=<string>] [--v3-user=<username>] [--v3-auth-key=<key>] [--v3-priv-key=<key>] [--v3-auth-proto=<%s>] [--v3-priv-proto=<%s>] [--context=<string>] [--use-getbulk] [--getbulk-repetitions=<number>] [--agent-udpv4-endpoint=<X.X.X.X:NNNNN>] [--agent-udpv6-endpoint=<[X:X:..X]:NNNNN>] [--agent-unix-endpoint=</path/to/named/pipe>] [--start-oid=<OID>] [--stop-oid=<OID>] [--output-file=<filename>] [--variation-modules-dir=<dir>] [--variation-module=<module>] [--variation-module-options=<args>]' % (sys.argv[0], '|'.join([ x for x in authProtocols if x != 'NONE' ]), '|'.join([ x for x in privProtocols if x != 'NONE' ]))
+helpMessage = """\
+Usage: %s [--help]
+    [--version]
+    [--debug=<%s>]
+    [--logging-method=<stdout|stderr|syslog|file>[:args>]]
+    [--version=<1|2c|3>]
+    [--community=<string>]
+    [--v3-user=<username>]
+    [--v3-auth-key=<key>]
+    [--v3-auth-proto=<%s>]
+    [--v3-priv-key=<key>]
+    [--v3-priv-proto=<%s>]
+    [--context=<string>]
+    [--use-getbulk]
+    [--getbulk-repetitions=<number>]
+    [--agent-udpv4-endpoint=<X.X.X.X:NNNNN>]
+    [--agent-udpv6-endpoint=<[X:X:..X]:NNNNN>]
+    [--agent-unix-endpoint=</path/to/named/pipe>]
+    [--start-oid=<OID>] [--stop-oid=<OID>]
+    [--output-file=<filename>]
+    [--variation-modules-dir=<dir>]
+    [--variation-module=<module>]
+    [--variation-module-options=<args>]""" % (
+        sys.argv[0],
+        '|'.join([ x for x in debug.flagMap.keys() if x != 'mibview' ]),
+        '|'.join([ x for x in authProtocols if x != 'NONE' ]),
+        '|'.join([ x for x in privProtocols if x != 'NONE' ])
+    )
 
 log.setLogger('snmprec', 'stdout')
 
 try:
-    opts, params = getopt.getopt(sys.argv[1:], 'h',
-        ['help', 'debug=', 'logging-method=', 'quiet', 'v1', 'v2c', 'v3', 'version=', 'community=', 'v3-user=', 'v3-auth-key=', 'v3-priv-key=', 'v3-auth-proto=', 'v3-priv-proto=', 'context=', 'use-getbulk', 'getbulk-repetitions=', 'agent-address=', 'agent-port=', 'agent-udpv4-endpoint=', 'agent-udpv6-endpoint=', 'agent-unix-endpoint=', 'start-oid=', 'stop-oid=', 'output-file=', 'variation-modules-dir=', 'variation-module=', 'variation-module-options=']
+    opts, params = getopt.getopt(sys.argv[1:], 'hv',
+        ['help', 'version', 'debug=', 'logging-method=', 'quiet', 'v1', 'v2c', 'v3', 'version=', 'community=', 'v3-user=', 'v3-auth-key=', 'v3-priv-key=', 'v3-auth-proto=', 'v3-priv-proto=', 'context=', 'use-getbulk', 'getbulk-repetitions=', 'agent-address=', 'agent-port=', 'agent-udpv4-endpoint=', 'agent-udpv6-endpoint=', 'agent-unix-endpoint=', 'start-oid=', 'stop-oid=', 'output-file=', 'variation-modules-dir=', 'variation-module=', 'variation-module-options=']
         )
 except Exception:
     sys.stderr.write('ERROR: %s\r\n%s\r\n' % (sys.exc_info()[1], helpMessage))
@@ -84,7 +110,24 @@ if params:
 
 for opt in opts:
     if opt[0] == '-h' or opt[0] == '--help':
-        sys.stderr.write('SNMP Simulator version %s, written by Ilya Etingof <ilya@glas.net>\r\nSoftware documentation and support at http://snmpsim.sf.net\r\n%s\r\n' % (__version__, helpMessage))
+        sys.stderr.write("""\
+Synopsis:
+  SNMP Agents Recording tool. Queries specified Agent, stores response
+  data in data files for subsequent playback by SNMP Simulation tool.
+  Can store a series of recordings for a more dynamic playback.
+Documentation:
+  http://snmpsim.sourceforge.net/snapshotting.html
+%s
+""" % helpMessage)
+        sys.exit(-1)
+    if opt[0] == '-v' or opt[0] == '--version':
+        import snmpsim, pysnmp, pyasn1
+        sys.stderr.write("""\
+SNMP Simulator version %s, written by Ilya Etingof <ilya@glas.net>
+Using foundation libraries: pysnmp %s, pyasn1 %s.
+Software documentation and support at http://snmpsim.sf.net
+%s
+""" % (snmpsim.__version__, hasattr(pysnmp, '__version__') and pysnmp.__version__ or 'unknown', hasattr(pyasn1, '__version__') and pyasn1.__version__ or 'unknown', helpMessage))
         sys.exit(-1)
     elif opt[0] == '--debug':
         debug.setLogger(debug.Debug(opt[1]))
@@ -95,7 +138,7 @@ for opt in opts:
             sys.stderr.write('%s\r\n%s\r\n' % (sys.exc_info()[1], helpMessage))
             sys.exit(-1)
     elif opt[0] == '--quiet':
-       log.setLogger('snmprec', 'null') 
+        log.setLogger('snmprec', 'null') 
     elif opt[0] == '--v1':
         snmpVersion = 0
     elif opt[0] == '--v2c':
