@@ -15,7 +15,7 @@ else:
     from hashlib import md5
 from pyasn1.type import univ
 from pyasn1.codec.ber import encoder, decoder
-from pyasn1.compat.octets import octs2str, str2octs, int2oct
+from pyasn1.compat.octets import str2octs
 from pyasn1.error import PyAsn1Error
 from pysnmp.entity import engine, config
 from pysnmp.entity.rfc3413 import cmdrsp, context
@@ -34,7 +34,6 @@ from pysnmp.smi.error import MibOperationError
 from pysnmp.proto import rfc1902, rfc1905, api
 from pysnmp import error
 from pysnmp import debug
-from snmpsim import __version__
 from snmpsim.error import SnmpsimError, NoDataNotification
 from snmpsim import confdir, log, daemon
 from snmpsim.record import dump, mvc, sap, walk, snmprec
@@ -54,9 +53,6 @@ v3PrivProtos = {}
 pidFile = '/var/run/snmpsim/snmpsimd.pid'
 foregroundFlag = True
 procUser = procGroup = None
-agentUDPv4Endpoints = []
-agentUDPv6Endpoints = []
-agentUNIXEndpoints = []
 variationModulesOptions = {}
 variationModules = {}
 
@@ -75,12 +71,69 @@ privProtocols = {
   'AES256': config.usmAesCfb256Protocol,
   'NONE': config.usmNoPrivProtocol
 }
+
+# Transport endpoints collection
+
+class TransportEndpointsBase:
+    def __init__(self):
+        self.__endpoints = []
+
+    def add(self, addr):
+        self.__endpoints.append(
+            self._addEndpoint(addr)
+        )
+
+    def load(self, filename):
+        for rec in open(filename).read().split():
+            self.add(rec)
+        
+    def _addEndpoint(self, addr): raise NotImplementedError()
+
+    def __len__(self): return len(self.__endpoints)
+    def __getitem__(self, i): return self.__endpoints[i]
+
+class IPv4TransportEndpoints(TransportEndpointsBase):
+    def _addEndpoint(self, addr):
+        f = lambda h,p=161: (h, int(p))
+        try:
+            h, p = f(*addr.split(':'))
+        except:
+            raise SnmpsimError('improper/busy IPv4/UDP endpoint %s' % addr)
+        return udp.UdpTransport().openServerMode((h, p)), addr
+
+agentUDPv4Endpoints = IPv4TransportEndpoints()
  
+class IPv6TransportEndpoints(TransportEndpointsBase):
+    def _addEndpoint(self, addr):
+        if not udp6:
+            raise SnmpsimError('This system does not support UDP/IP6')
+        if addr.find(']:') != -1 and addr[0] == '[':
+            h, p = addr.split(']:')
+            try:
+                h, p = h[1:], int(p)
+            except:
+                raise SnmpsimError('improper/busy IPv6/UDP endpoint %s' % addr)
+        elif addr[0] == '[' and addr[-1] == ']':
+            h, p = addr[1:-1], 161
+        else:
+            h, p = addr, 161
+        return udp6.Udp6Transport().openServerMode((h, p)), addr
+ 
+agentUDPv6Endpoints = IPv6TransportEndpoints()
+
+class UnixTransportEndpoints(TransportEndpointsBase):
+    def _addEndpoint(self, addr):
+        if not unix:
+            raise SnmpsimError('This system does not support UNIX domain sockets')
+        return unix.UnixTransport().openServerMode(addr), addr
+
+agentUNIXEndpoints = UnixTransportEndpoints()
+
 # Extended snmprec record handler
 
 class SnmprecRecord(snmprec.SnmprecRecord):
     def evaluateValue(self, oid, tag, value, **context):
-        # Interpolation module reference
+        # Variation module reference
         if ':' in tag:
             modName, tag = tag[tag.index(':')+1:], tag[:tag.index(':')]
         else:
@@ -116,12 +169,12 @@ class SnmprecRecord(snmprec.SnmprecRecord):
 
         if not modName:
             if 'dataValidation' in context:
-                    snmprec.SnmprecRecord.evaluateValue(
-                        self, oid, tag, value, **context
-                    )
+                snmprec.SnmprecRecord.evaluateValue(
+                    self, oid, tag, value, **context
+                )
 
             if not context['nextFlag'] and not context['exactMatch'] or \
-               context['setFlag']:
+                   context['setFlag']:
                 return context['origOid'], tag, context['errorStatus']
 
         if not hasattr(value, 'tagSet'):  # not already a pyasn1 object
@@ -156,7 +209,7 @@ recordSet = {
 }
 
 class AbstractLayout:
-  layout = '?'
+    layout = '?'
 
 # Data text file and OID index
 
@@ -228,18 +281,7 @@ class DataFile(AbstractLayout):
             varsRemaining -= 1
 
             line = text.readline()  # matched line
-           
-            # case of a subtree on the last line 
-            if not exactMatch and not line:
-                offset -= 1
-                while offset > 0:
-                    offset -= 1
-                    text.seek(offset)
-                    if text.read(1) == '\n':
-                        break
-                text.seek(offset)
-                line = text.readline()  # last line
-
+ 
             while True:
                 if exactMatch:
                     if nextFlag and not subtreeFlag:
@@ -251,23 +293,29 @@ class DataFile(AbstractLayout):
                             except KeyError:
                                 log.msg('data error for %s at %s, index broken?' % (self, _nextOid))
                                 line = ''  # fatal error
-                                break
-                            subtreeFlag = int(subtreeFlag)
-                        line = _nextLine
+                            else:
+                                subtreeFlag = int(subtreeFlag)
+                                line = _nextLine
+                        else:
+                            line = _nextLine
                 else: # search function above always rounds up to the next OID
                     if line:
                         _oid, _  = self.__textParser.evaluate(
                             line, oidOnly=True
                         )
-                        try:
-                            _, _, _prevOffset = self.__recordIndex.lookup(str(_oid)).split(str2octs(','))
-                        except KeyError:
-                            log.msg('data error for %s at %s, index broken?' % (self, _oid))
-                            line = ''  # fatal error
-                            break
+                    else:  # eom
+                        _oid = 'last'
+
+                    try:
+                        _, _, _prevOffset = self.__recordIndex.lookup(str(_oid)).split(str2octs(','))
+                    except KeyError:
+                        log.msg('data error for %s at %s, index broken?' % (self, _oid))
+                        line = ''  # fatal error
+                    else:
                         _prevOffset = int(_prevOffset)
 
-                        if _prevOffset >= 0:  # previous line serves a subtree
+                        # previous line serves a subtree?
+                        if _prevOffset >= 0:
                             text.seek(_prevOffset)
                             _prevLine = text.readline()
                             _prevOid, _ = self.__textParser.evaluate(
@@ -429,10 +477,10 @@ if not v2cArch:
             except error.PySnmpError:
                 pass
             else:
-                log.msg('Using %s selected by candidate %s; transport ID %s, source address %s, community name "%s"' % (mibInstrum, candidate, univ.ObjectIdentifier(transportDomain), transportAddress[0], communityName))
+                log.msg('Using %s selected by candidate %s; transport ID %s, source address %s, context name "%s"' % (mibInstrum, candidate, univ.ObjectIdentifier(transportDomain), transportAddress[0], probedContextName))
                 return probedContextName
-        else:
-            log.msg('Using %s selected by contextName "%s", transport ID %s, source address %s' % (self.snmpContext.getMibInstrum(contextName), contextName, univ.ObjectIdentifier(transportDomain), transportAddress[0]))
+
+        log.msg('Using %s selected by contextName "%s", transport ID %s, source address %s' % (self.snmpContext.getMibInstrum(contextName), contextName, univ.ObjectIdentifier(transportDomain), transportAddress[0]))
 
         return contextName
 
@@ -478,11 +526,41 @@ if not v2cArch:
 
 # main script body starts here
 
-helpMessage = 'Usage: %s [--help] [--version ] [--debug=<category>] [--daemonize] [--process-user=<uname>] [--process-group=<gname>] [--logging-method=<method|?>] [--data-dir=<dir>] [--cache-dir=<dir>] [--force-index-rebuild] [--validate-data] [--variation-modules-dir=<dir>] [--variation-module-options=<module[=alias][:args]>] [--agent-udpv4-endpoint=<X.X.X.X:NNNNN>] [--agent-udpv6-endpoint=<[X:X:..X]:NNNNN>] [--agent-unix-endpoint=</path/to/named/pipe>] [--v2c-arch] [--v3-only] [--v3-user=<username>] [--v3-auth-key=<key>] [--v3-auth-proto=<%s>] [--v3-priv-key=<key>] [--v3-priv-proto=<%s>]' % (sys.argv[0], '|'.join(authProtocols), '|'.join(privProtocols))
+helpMessage = """\
+Usage: %s [--help]
+    [--version ]
+    [--debug=<%s>]
+    [--daemonize]
+    [--process-user=<uname>] [--process-group=<gname>]
+    [--logging-method=<stdout|stderr|syslog|file>[:args>]]
+    [--data-dir=<dir>]
+    [--cache-dir=<dir>]
+    [--variation-modules-dir=<dir>]
+    [--variation-module-options=<module[=alias][:args]>] 
+    [--force-index-rebuild]
+    [--validate-data]
+    [--agent-udpv4-endpoint=<X.X.X.X:NNNNN>]
+    [--agent-udpv6-endpoint=<[X:X:..X]:NNNNN>]
+    [--agent-unix-endpoint=</path/to/named/pipe>]
+    [--agent-udpv4-endpoints-list=<file>]
+    [--agent-udpv6-endpoints-list=<file>]
+    [--agent-unix-endpoints-list=<file>]
+    [--v2c-arch]
+    [--v3-only]
+    [--v3-user=<username>]
+    [--v3-auth-key=<key>]
+    [--v3-auth-proto=<%s>]
+    [--v3-priv-key=<key>]
+    [--v3-priv-proto=<%s>]""" % (
+        sys.argv[0],
+        '|'.join([ x for x in debug.flagMap.keys() if x != 'mibview' ]),
+        '|'.join(authProtocols),
+        '|'.join(privProtocols)
+    )
 
 try:
-    opts, params = getopt.getopt(sys.argv[1:], 'h',
-        ['help', 'debug=', 'daemonize', 'process-user=', 'process-group=', 'logging-method=', 'device-dir=', 'data-dir=', 'cache-dir=', 'force-index-rebuild', 'validate-device-data', 'validate-data', 'variation-modules-dir=', 'variation-module-options=', 'agent-address=', 'agent-port=', 'agent-udpv4-endpoint=', 'agent-udpv6-endpoint=', 'agent-unix-endpoint=', 'v2c-arch', 'v3-only', 'v3-user=', 'v3-auth-key=', 'v3-auth-proto=', 'v3-priv-key=', 'v3-priv-proto=']
+    opts, params = getopt.getopt(sys.argv[1:], 'hv',
+        ['help', 'version', 'debug=', 'daemonize', 'process-user=', 'process-group=', 'logging-method=', 'device-dir=', 'data-dir=', 'cache-dir=', 'force-index-rebuild', 'validate-device-data', 'validate-data', 'variation-modules-dir=', 'variation-module-options=', 'agent-address=', 'agent-port=', 'agent-udpv4-endpoint=', 'agent-udpv6-endpoint=', 'agent-unix-endpoint=', 'agent-udpv4-endpoints-list=', 'agent-udpv6-endpoints-list=', 'agent-unix-endpoints-list=', 'v2c-arch', 'v3-only', 'v3-user=', 'v3-auth-key=', 'v3-auth-proto=', 'v3-priv-key=', 'v3-priv-proto=']
         )
 except Exception:
     sys.stderr.write('ERROR: %s\r\n%s\r\n' % (sys.exc_info()[1], helpMessage))
@@ -495,9 +573,25 @@ if params:
 log.setLogger('snmpsimd', 'stdout')
 
 for opt in opts:
-    if opt[0] == '-h' or opt[0] == '--help' or \
-       opt[0] == '-v' or opt[0] == '--version':
-        sys.stderr.write('SNMP Simulator version %s, written by Ilya Etingof <ilya@glas.net>\r\nSoftware documentation and support at http://snmpsim.sf.net\r\n%s\r\n' % (__version__, helpMessage))
+    if opt[0] == '-h' or opt[0] == '--help':
+        sys.stderr.write("""\
+Synopsis:
+  SNMP Agents Simulation tool. Responds to SNMP requests, variate responses
+  based on transport addresses, SNMP community name or SNMPv3 context name.
+  Can implement highly complex behavior through variation modules.
+Documentation:
+  http://snmpsim.sourceforge.net/simulating-agents.html
+%s
+""" % helpMessage)
+        sys.exit(-1)
+    if opt[0] == '-v' or opt[0] == '--version':
+        import snmpsim, pysnmp, pyasn1
+        sys.stderr.write("""\
+SNMP Simulator version %s, written by Ilya Etingof <ilya@glas.net>
+Using foundation libraries: pysnmp %s, pyasn1 %s.
+Software documentation and support at http://snmpsim.sf.net
+%s
+""" % (snmpsim.__version__, hasattr(pysnmp, '__version__') and pysnmp.__version__ or 'unknown', hasattr(pyasn1, '__version__') and pyasn1.__version__ or 'unknown', helpMessage))
         sys.exit(-1)
     elif opt[0] == '--debug':
         debug.setLogger(debug.Debug(opt[1]))
@@ -538,37 +632,41 @@ for opt in opts:
             variationModulesOptions[modName] = []
         variationModulesOptions[modName].append((alias, args))
     elif opt[0] == '--agent-udpv4-endpoint':
-        f = lambda h,p=161: (h, int(p))
         try:
-            agentUDPv4Endpoints.append((udp.UdpTransport().openServerMode(f(*opt[1].split(':'))), opt[1]))
+            agentUDPv4Endpoints.add(opt[1])
         except:
-            sys.stderr.write('ERROR: improper/busy IPv4/UDP endpoint %s\r\n%s\r\n' % (opt[1], helpMessage))
+            sys.stderr.write('ERROR: %s\r\n%s\r\n' % (sys.exc_info()[1], helpMessage))
             sys.exit(-1)
     elif opt[0] == '--agent-udpv6-endpoint':
-        if not udp6:
-            sys.stderr.write('This system does not support UDP/IP6\r\n')
+        try:
+            agentUDPv6Endpoints.add(opt[1])
+        except:
+            sys.stderr.write('ERROR: %s\r\n%s\r\n' % (sys.exc_info()[1], helpMessage))
             sys.exit(-1)
-        if opt[1].find(']:') != -1 and opt[1][0] == '[':
-            h, p = opt[1].split(']:')
-            try:
-                h, p = h[1:], int(p)
-            except:
-                sys.stderr.write('ERROR: improper/busy IPv6/UDP endpoint %s\r\n%s\r\n' % (opt[1], helpMessage))
-                sys.exit(-1)
-        elif opt[1][0] == '[' and opt[1][-1] == ']':
-            h, p = opt[1][1:-1], 161
-        else:
-            h, p = opt[1], 161
-        agentUDPv6Endpoints.append(
-            (udp6.Udp6Transport().openServerMode((h, p)), opt[1])
-        )
     elif opt[0] == '--agent-unix-endpoint':
-        if not unix:
-            sys.stderr.write('This system does not support UNIX domain sockets\r\n')
+        try:
+            agentUNIXEndpoints.add(opt[1])
+        except:
+            sys.stderr.write('ERROR: %s\r\n%s\r\n' % (sys.exc_info()[1], helpMessage))
+            sys.exit(-1)
+    elif opt[0] == '--agent-udpv4-endpoints-list':
+        try:
+            agentUDPv4Endpoints.load(opt[1])
+        except:
+            sys.stderr.write('ERROR: %s\r\n%s\r\n' % (sys.exc_info()[1], helpMessage))
+            sys.exit(-1)
+    elif opt[0] == '--agent-udpv6-endpoints-list':
+        try:
+            agentUDPv6Endpoints.load(opt[1])
+        except:
+            sys.stderr.write('ERROR: %s\r\n%s\r\n' % (sys.exc_info()[1], helpMessage))
+            sys.exit(-1)
+    elif opt[0] == '--agent-unix-endpoints-list':
+        try:
+            agentUNIXEndpoints.load(opt[1])
+        except:
+            sys.stderr.write('ERROR: %s\r\n%s\r\n' % (sys.exc_info()[1], helpMessage))
             sys.exit(-1)
-        agentUNIXEndpoints.append(
-            (unix.UnixTransport().openServerMode(opt[1]), opt[1])
-        )
     elif opt[0] == '--agent-address':
         sys.stderr.write('ERROR: use --agent-udpv4-endpoint=%s option instead of --agent-address\r\n%s\r\n' % (opt[1], helpMessage))
         sys.exit(-1)
@@ -610,9 +708,9 @@ for v3User in v3Users:
         v3PrivKeys[v3User] = None
         v3PrivProtos[v3User] = 'NONE'
     if authProtocols[v3AuthProtos[v3User]] == config.usmNoAuthProtocol and \
-        privProtocols[v3PrivProtos[v3User]] != config.usmNoPrivProtocol:
-            sys.stderr.write('ERROR: privacy impossible without authentication\r for USM user %s\n%s\r\n', (v3User, helpMessage))
-            sys.exit(-1)
+            privProtocols[v3PrivProtos[v3User]] != config.usmNoPrivProtocol:
+        sys.stderr.write('ERROR: privacy impossible without authentication\r for USM user %s\n%s\r\n', (v3User, helpMessage))
+        sys.exit(-1)
 
 if not v3Users:
     v3Users = [ 'simulator' ]
@@ -622,10 +720,10 @@ if not v3Users:
     v3PrivProtos[v3Users[0]] = 'DES'
 
 if not agentUDPv4Endpoints and \
-   not agentUDPv6Endpoints and \
-   not agentUNIXEndpoints:
-        sys.stderr.write('ERROR: agent endpoint address(es) not specified\r\n%s\r\n' % helpMessage)
-        sys.exit(-1)
+        not agentUDPv6Endpoints and \
+        not agentUNIXEndpoints:
+    sys.stderr.write('ERROR: agent endpoint address(es) not specified\r\n%s\r\n' % helpMessage)
+    sys.exit(-1)
 
 try:
     daemon.dropPrivileges(procUser, procGroup)
@@ -718,8 +816,8 @@ else:
 
 if variationModules:
     log.msg('Initializing variation modules...')
-    for name, contexts in variationModules.items():
-        body = contexts[0]
+    for name, modulesContexts in variationModules.items():
+        body = modulesContexts[0]
         for x in ('init', 'variate', 'shutdown'):
             if x not in body:
                 log.msg('ERROR: missing %s handler in %s!' % (x, name))
@@ -791,22 +889,24 @@ del _mibInstrums
 del _dataFiles
 
 if v2cArch:
-    def getBulkHandler(varBinds, nonRepeaters, maxRepetitions, readNextVars):
-        if nonRepeaters < 0: nonRepeaters = 0
-        if maxRepetitions < 0: maxRepetitions = 0
-        N = min(nonRepeaters, len(varBinds))
+    def getBulkHandler(reqVarBinds, nonRepeaters, maxRepetitions, readNextVars):
+        N = min(int(nonRepeaters), len(reqVarBinds))
         M = int(maxRepetitions)
-        R = max(len(varBinds)-N, 0)
-        if nonRepeaters:
-            rspVarBinds = readNextVars(varBinds[:int(nonRepeaters)])
+        R = max(len(reqVarBinds)-N, 0)
+        if R: M = min(M, 100/R)  # maxVarBinds
+
+        if N:
+            rspVarBinds = readNextVars(reqVarBinds[:N])
         else:
             rspVarBinds = []
-        if M and R:
-            for i in range(N,  R):
-                varBind = varBinds[i]
-                for r in range(1, M):
-                    rspVarBinds.extend(readNextVars((varBind,)))
-                    varBind = rspVarBinds[-1]
+
+        varBinds = reqVarBinds[-R:]
+        while M and R:
+            rspVarBinds.extend(
+                readNextVars(varBinds)
+            )
+            varBinds = rspVarBinds[-R:]
+            M = M - 1
 
         return rspVarBinds
  
