@@ -1,13 +1,11 @@
 # SNMP Simulator, http://snmpsim.sourceforge.net
 # Managed value variation module
 # Send SNMP Notification
-
 import sys
 from pysnmp.entity.rfc3413.oneliner import ntforg
 from pysnmp.proto import rfc1902
 from snmpsim.grammar.snmprec import SnmprecGrammar
-
-settingsCache = {}
+from snmpsim import error, log
 
 def init(snmpEngine, **context):
     global ntfOrg
@@ -35,17 +33,17 @@ def _cbFun(sendRequestHandle,
           cbCtx):
     if errorIndication or errorStatus:
         oid, value = cbCtx
-        sys.stdout.write('notification: for %s=%r failed with errorIndication %s, errorStatus %s\r\n' % (oid, value, errorIndication, errorStatus))
+        log.msg('notification: for %s=%r failed with errorIndication %s, errorStatus %s' % (oid, value, errorIndication, errorStatus))
 
 def variate(oid, tag, value, **context):
     if ntfOrg is None:
-        raise Exception('variation module not initialized')
+        raise error.SnmpsimError('variation module not initialized')
 
     if not context['nextFlag'] and not context['exactMatch']:
         return context['origOid'], tag, context['errorStatus']
 
-    if oid not in settingsCache:
-        settingsCache[oid] = dict([ x.split('=') for x in value.split(',') ])
+    if 'settings' not in recordContext:
+        recordContext['settings'] = dict([ x.split('=') for x in value.split(',') ])
         for k,v in ( ('op', 'set'),
                      ('community', 'public'),
                      ('authkey', None),
@@ -56,17 +54,17 @@ def variate(oid, tag, value, **context):
                      ('port', '162'),
                      ('ntftype', 'trap'),
                      ('trapoid', '1.3.6.1.6.3.1.1.5.1') ):
-            settingsCache[oid].setdefault(k, v)
+            recordContext['settings'].setdefault(k, v)
 
-        if 'hexvalue' in settingsCache[oid]:
-            settingsCache[oid]['value'] = [int(settingsCache[oid]['hexvalue'][x:x+2], 16) for x in range(0, len(settingsCache[oid]['hexvalue']), 2)]
+        if 'hexvalue' in recordContext['settings']:
+            recordContext['settings']['value'] = [int(recordContext['settings']['hexvalue'][x:x+2], 16) for x in range(0, len(recordContext['settings']['hexvalue']), 2)]
 
-        if 'vlist' in settingsCache[oid]:
+        if 'vlist' in recordContext['settings']:
             vlist = {}
-            settingsCache[oid]['vlist'] = settingsCache[oid]['vlist'].split(':')
-            while settingsCache[oid]['vlist']:
-                o,v = settingsCache[oid]['vlist'][:2]
-                settingsCache[oid]['vlist'] = settingsCache[oid]['vlist'][2:]
+            recordContext['settings']['vlist'] = recordContext['settings']['vlist'].split(':')
+            while recordContext['settings']['vlist']:
+                o,v = recordContext['settings']['vlist'][:2]
+                recordContext['settings']['vlist'] = recordContext['settings']['vlist'][2:]
                 v = SnmprecGrammar.tagMap[tag](v)
                 if o not in vlist:
                     vlist[o] = set()
@@ -75,10 +73,10 @@ def variate(oid, tag, value, **context):
                 elif o in ('lt', 'gt'):
                     vlist[o] = v
                 else:
-                    sys.stdout.write('delay: bad vlist syntax: %s\r\n' % settingsCache[oid]['vlist'])
-            settingsCache[oid]['vlist'] = vlist
+                    log.msg('notification: bad vlist syntax: %s' % recordContext['settings']['vlist'])
+            recordContext['settings']['vlist'] = vlist
 
-    args = settingsCache[oid]
+    args = recordContext['settings']
    
     if context['setFlag'] and 'vlist' in args:
         if 'eq' in args['vlist'] and  \
@@ -94,7 +92,7 @@ def variate(oid, tag, value, **context):
             return oid, tag, context['origValue']
 
     if args['op'] not in ('get', 'set', 'any', '*'):
-        sys.stdout.write('notification: unknown SNMP request type configured: %s\r\n' % args['op'])
+        log.msg('notification: unknown SNMP request type configured: %s' % args['op'])
         return context['origOid'], tag, context['errorStatus']
 
     if args['op'] == 'get' and not context['setFlag'] or \
@@ -110,7 +108,7 @@ def variate(oid, tag, value, **context):
             elif args['authproto'] == 'none':
                 authProtocol = ntforg.usmNoAuthProtocol
             else:
-                sys.stdout.write('notification: unknown auth proto %s\r\n' % args['authproto'])
+                log.msg('notification: unknown auth proto %s' % args['authproto'])
                 return context['origOid'], tag, context['errorStatus']
             if args['privproto'] == 'des':
                 privProtocol = ntforg.usmDESPrivProtocol
@@ -119,15 +117,15 @@ def variate(oid, tag, value, **context):
             elif args['privproto'] == 'none':
                 privProtocol = ntforg.usmNoPrivProtocol
             else:
-                sys.stdout.write('notification: unknown privacy proto %s\r\n' % args['privproto'])
+                log.msg('notification: unknown privacy proto %s' % args['privproto'])
                 return context['origOid'], tag, context['errorStatus']
             authData = ntforg.UsmUserData(args['user'], args['authkey'], args['privkey'], authProtocol=authProtocol, privProtocol=privProtocol)
         else:
-            sys.stdout.write('notification: unknown SNMP version %s\r\n' % args['version'])
+            log.msg('notification: unknown SNMP version %s' % args['version'])
             return context['origOid'], tag, context['errorStatus']
 
         if 'host' not in args:
-            sys.stdout.write('notification: target hostname not configured for OID\r\n' % (oid,))
+            log.msg('notification: target hostname not configured for OID' % (oid,))
             return context['origOid'], tag, context['errorStatus']
 
         if args['proto'] == 'udp':
@@ -135,7 +133,7 @@ def variate(oid, tag, value, **context):
         elif args['proto'] == 'udp6':
             target = ntforg.Udp6TransportTarget((args['host'], int(args['port'])))
         else:
-            sys.stdout.write('notification: unknown transport %s\r\n' % args['proto'])
+            log.msg('notification: unknown transport %s' % args['proto'])
             return context['origOid'], tag, context['errorStatus']
 
         varBinds = []
@@ -172,6 +170,8 @@ def variate(oid, tag, value, **context):
             varBinds, cbInfo=(_cbFun, (oid, value))
         )
 
+        log.msg('notification: sending Notification to %s with credentials %s' % (authData, target))
+
     if context['setFlag'] or 'value' not in args:
         return oid, tag, context['origValue']
     else:
